﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.Serialization;
using System.ComponentModel.DataAnnotations;
using Humanizer;
using KiwiCore.Security;

namespace KiwiCore.Models
{
    [DataContract]
    public class Adjustment : BaseModel, IListItem, IValidatableObject
    {
        

        public Adjustment()
        {
            AdjustmentDetails = new List<AdjustmentDetail>();
            Attachments = new List<Attachment>();
            UserDelegators = new List<AdvUser>();
            AdHocApprovers = new List<string>();
            SetupIssues = new List<SetupIssue>();
            Acl = new AdjustmentAcl();
        }

        [DataMember(Name = "adjustment_id", EmitDefaultValue = true)]
        public int AdjustmentId { get; set; }

        [DataMember(Name = "adjustment_description", EmitDefaultValue = true)]
        [Required(ErrorMessage = "Description cannot be blank")]
        public string Description { get; set; }

        [DataMember(Name = "order_id", EmitDefaultValue = true)]
        public int OrderId { get; set; }

        [DataMember(Name = "total_adjustment_amount", EmitDefaultValue = true)]
        public decimal TotalAdjustmentAmount
        {
            get { return (AdjustmentDetails?.Sum(dtl => dtl.AdjustmentAmount) ?? 0); }
        }
        /// <summary>
        /// Get the total adjustment amount in a '$xx.xx [CR|DR]' format.
        /// </summary>
        [DataMember(Name = "total_adjustment_amount_ext")]
        public string TotalAdjustmentAmountExt => $"{TotalAdjustmentAmount.ToString("C")} {(IsCredit ? "CR" : "DR")}";

        [DataMember(Name = "total_actual_amount", EmitDefaultValue = true)]
        public decimal TotalActualAmount
        {
            get { return AdjustmentDetails.Sum(dtl => dtl.ActualAmount); }
        }

        [DataMember(Name = "customer", EmitDefaultValue = true)]
        public Customer Customer { get; set; }

        [DataMember(Name = "adjustment_notes", EmitDefaultValue = true)]
        public List<Note> Notes { get; set; }

        [DataMember(Name = "adjustment_type")]
        public Enums.AdjustmentType AdjustmentType { get; set; }

        [DataMember(Name = "adjustment_type_name")]
        public string AdjustmentTypeName => AdjustmentType.Humanize(LetterCasing.Title);

        [DataMember(Name = "adjustment_status", EmitDefaultValue = true)]
        public Enums.AdjustmentStatus Status { get; set; }

        [DataMember(Name = "adjustment_status_name", EmitDefaultValue = true)]
        public string StatusName => Status.Humanize(LetterCasing.Title);

        [DataMember(Name = "po_number", EmitDefaultValue = true)]
        public string PoNumber { get; set; }

        [DataMember(Name = "is_credit", EmitDefaultValue = true)]
        [Required(ErrorMessage = "IsCredit is required")]
        public bool IsCredit { get; set; }

        [DataMember(Name = "adjustment_details", EmitDefaultValue = true)]
        [AtLeastOneItem(ErrorMessage = "At least one Adjustment Detail is required.")]
        public List<AdjustmentDetail> AdjustmentDetails { get; set; }

        [DataMember(Name = "attachments", EmitDefaultValue = true)]
        public List<Attachment> Attachments { get; set; }

        [DataMember(Name = "adjustment_author", EmitDefaultValue = true)]
        public AdvUser Author { get; set; }

        [DataMember(Name = "site")]
        public Site Site { get; set; }

        [DataMember(Name = "ad_hoc_approvers")]
        public List<string> AdHocApprovers { get; set; }

        /// <summary>
        /// Get or set the list of users that the current user is a delegate for.  The current user will be acting on behalf of the users in this list.
        /// </summary>
        [DataMember(Name = "user_delegators")]
        public List<AdvUser> UserDelegators { get; set; }
        /// <summary>
        /// Get value indicating if the attachments list is considered read-only (i.e. no adds or deletes allowed).
        /// </summary>
        [DataMember(Name = "attachments_readonly")]
        public bool AttachmentsReadOnly { get; set; }

        /// <summary>
        /// Get or set the period that the Adjustment was created in.
        /// </summary>
        [DataMember(Name = "create_period")]
        public FiscalPeriod CreatePeriod { get; set; }
        /// <summary>
        /// Get or set the period that the Adjustment was batched in.
        /// </summary>
        [DataMember(Name = "batch_period")]
        public FiscalPeriod BatchPeriod { get; set; }

        [DataMember(Name = "setup_issues")]
        public List<SetupIssue> SetupIssues { get; set; }

        [DataMember(Name = "acl")]
        public AdjustmentAcl Acl { get; set; }

        [DataMember(Name = "status_readonly")]
        public bool StatusReadOnly
        {
            get
            {
                //Read-only when (Status <> "New" AND Status <> "Returned")
                return (Status != Enums.AdjustmentStatus.New &&
                        Status != Enums.AdjustmentStatus.Returned);
            }
        }

        #region Implementation of IListItem

        public int Value { get { return AdjustmentId; } set { AdjustmentId = value; } }

        public string Text { get; set; }

        #endregion

        #region Implementation of IValidatableObject
        public virtual IEnumerable<ValidationResult> Validate(ValidationContext validationContext)
        {
            List<ValidationResult> results = new List<ValidationResult>();

            switch (AdjustmentType)
            {
                //Standard Adjustment Validations
                case Enums.AdjustmentType.Standard: 
                    //Author
                    results.AddRange(Author.Validate(validationContext));

                    //Customer
                    results.AddRange(Customer.Validate(validationContext));

                    //AdjustmentDetails
                    foreach (var adjDetail in AdjustmentDetails)
                    {
                        results.AddRange(adjDetail.Validate(validationContext));

                        if (AdjustmentId > 0)
                        {
                            //GLAccount - required when it's past Accounting Review (i.e. Pending) or if it's Batched or if it's Approved (all parties have approved)
                            if (Status == Enums.AdjustmentStatus.Pending || Status == Enums.AdjustmentStatus.Batched || Status == Enums.AdjustmentStatus.Approved)
                            {
                                int phxGlAccountId;
                                bool isN = int.TryParse((adjDetail.GlAccount == null) ? "-1" : adjDetail.GlAccount.GlAccountId, out phxGlAccountId);

                                // If the GL Account ID is -2, it is an APT site, so we only require an object
                                if (phxGlAccountId != -2 && phxGlAccountId <= 0)
                                    results.Add(new ValidationResult("GL Account is required"));
                            }

                            //Salesperson Supervisor
                            if (Status != Enums.AdjustmentStatus.Rejected && Status != Enums.AdjustmentStatus.Returned)
                            {
                            if (adjDetail.SalesPerson == null || adjDetail.SalesPerson.SupervisorId <= 0)
                                results.Add(new ValidationResult(Constants.ErrorMessages.SalespersonSupervisorRequired));
                            }

                            //AdjustmentReviews - ensure there is a ReviewReason of Supervisor or we cannot route
                            if (Status == Enums.AdjustmentStatus.Pending || Status == Enums.AdjustmentStatus.Approved)
                            {
                                if (!adjDetail.AdjustmentReviews.Any(rev => rev.ReviewReason == Enums.ReviewReason.Supervisor))
                                    results.Add(new ValidationResult("Unable to route this Adjustment. Please notify the Site Administrator that there is no available Supervisor (not including the Author) to assign to."));
                            }
                        }
                    }
                    break;

                //Central Adjustment Validations
                case Enums.AdjustmentType.RFC:
                case Enums.AdjustmentType.Corp:
                    //AdjustmentDetails must be EXACTLY 1
                    if (AdjustmentDetails.Count != 1)
                        results.Add(new ValidationResult("A Central Adjustment must have EXACTLY 1 AdjustmentDetail"));
                    break;
            }

            return results;
        }
        #endregion

        public bool IsReviewPending(IKiwiPrincipal kiwUser, string advUserId)
        {
            foreach (AdjustmentDetail adjDetail in AdjustmentDetails)
            {
                foreach (AdjustmentReview adjReview in adjDetail.AdjustmentReviews)
                {
                    if ((adjReview.AssignedUser.UserId == advUserId || kiwUser.IsADelegate(Site.SiteId, adjReview.AssignedUser.NetName, true)) &&
                        adjReview.ReviewStatus == Enums.ReviewStatus.Pending)
                    {
                        return true;
                    }
                }
            }

            return false;
        }

        
    }
}